# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_static.ipynb.

# %% auto 0
__all__ = ['LOG_SAFETY', 'SSR_CHI2TEST', 'LAG_ORDER', 'DEFAULT_TEST', 'DEFAULT_GRANGER_CAUSALITY_TESTS', 'LOG2_FOLD',
           'SIGNED_CORRELATION', 'STANDARD_SCALER', 'KNOWN_TRANSFORMS', 'MOCK_GENES']

# %% ../nbs/00_static.ipynb 3
LOG_SAFETY = 1e-8

SSR_CHI2TEST = 'ssr_chi2test'
LAG_ORDER = 1

DEFAULT_TEST = SSR_CHI2TEST

DEFAULT_GRANGER_CAUSALITY_TESTS = dict(
    test      =  DEFAULT_TEST,
    lag_order =  LAG_ORDER,
    max_lag   = (LAG_ORDER, ) 
)


LOG2_FOLD = 'log2_fold'
SIGNED_CORRELATION= 'signed_correlation'
STANDARD_SCALER = 'standard_scaler'

KNOWN_TRANSFORMS = [LOG2_FOLD, SIGNED_CORRELATION, STANDARD_SCALER]

MOCK_GENES = 'wasf colq gpr1 chrm3 lmod2 tek kank3 oca2 taz map4k1'.split()
