# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/gnr',
                'doc_host': 'https://dsm-72.github.io',
                'git_url': 'https://github.com/dsm-72/gnr',
                'lib_path': 'gnr'},
  'syms': { 'gnr.fn': { 'gnr.fn.calculate_granger_causation': ('fn.html#calculate_granger_causation', 'gnr/fn.py'),
                        'gnr.fn.get_pval_from_granger_causality_tests': ('fn.html#get_pval_from_granger_causality_tests', 'gnr/fn.py'),
                        'gnr.fn.grangers_causation_matrix': ('fn.html#grangers_causation_matrix', 'gnr/fn.py'),
                        'gnr.fn.shift_trajectories': ('fn.html#shift_trajectories', 'gnr/fn.py')},
            'gnr.mock': { 'gnr.mock.df_trj_to_cells_x_tbins': ('mock.html#df_trj_to_cells_x_tbins', 'gnr/mock.py'),
                          'gnr.mock.df_trj_to_genes_x_tbins': ('mock.html#df_trj_to_genes_x_tbins', 'gnr/mock.py'),
                          'gnr.mock.make_dummy_cells': ('mock.html#make_dummy_cells', 'gnr/mock.py'),
                          'gnr.mock.make_dummy_genes': ('mock.html#make_dummy_genes', 'gnr/mock.py'),
                          'gnr.mock.make_mock_genes_x_tbins': ('mock.html#make_mock_genes_x_tbins', 'gnr/mock.py'),
                          'gnr.mock.make_mock_trajectories': ('mock.html#make_mock_trajectories', 'gnr/mock.py')},
            'gnr.op': { 'gnr.op.GrangerCausality': ('op.html#grangercausality', 'gnr/op.py'),
                        'gnr.op.GrangerCausality.__init__': ('op.html#grangercausality.__init__', 'gnr/op.py'),
                        'gnr.op.GrangerCausality.apply_log2_fold': ('op.html#grangercausality.apply_log2_fold', 'gnr/op.py'),
                        'gnr.op.GrangerCausality.apply_signed_correlation': ( 'op.html#grangercausality.apply_signed_correlation',
                                                                              'gnr/op.py'),
                        'gnr.op.GrangerCausality.apply_standard_scaler': ('op.html#grangercausality.apply_standard_scaler', 'gnr/op.py'),
                        'gnr.op.GrangerCausality.fit': ('op.html#grangercausality.fit', 'gnr/op.py'),
                        'gnr.op.GrangerCausality.fit_transform': ('op.html#grangercausality.fit_transform', 'gnr/op.py'),
                        'gnr.op.GrangerCausality.invert_scaler': ('op.html#grangercausality.invert_scaler', 'gnr/op.py'),
                        'gnr.op.GrangerCausality.plot_df_org': ('op.html#grangercausality.plot_df_org', 'gnr/op.py'),
                        'gnr.op.GrangerCausality.plot_df_res': ('op.html#grangercausality.plot_df_res', 'gnr/op.py'),
                        'gnr.op.GrangerCausality.process_fit_params': ('op.html#grangercausality.process_fit_params', 'gnr/op.py'),
                        'gnr.op.GrangerCausality.transform': ('op.html#grangercausality.transform', 'gnr/op.py')},
            'gnr.static': {},
            'gnr.utils': { 'gnr.utils._add_suffix': ('utils.html#_add_suffix', 'gnr/utils.py'),
                           'gnr.utils._prep_args_for_granger_causality_tests': ( 'utils.html#_prep_args_for_granger_causality_tests',
                                                                                 'gnr/utils.py'),
                           'gnr.utils._prep_vars_for_granger_causality_tests': ( 'utils.html#_prep_vars_for_granger_causality_tests',
                                                                                 'gnr/utils.py'),
                           'gnr.utils.add_non_const': ('utils.html#add_non_const', 'gnr/utils.py'),
                           'gnr.utils.add_stability': ('utils.html#add_stability', 'gnr/utils.py'),
                           'gnr.utils.apply_log2_fold': ('utils.html#apply_log2_fold', 'gnr/utils.py'),
                           'gnr.utils.apply_signed_correlation': ('utils.html#apply_signed_correlation', 'gnr/utils.py'),
                           'gnr.utils.apply_standard_scaler': ('utils.html#apply_standard_scaler', 'gnr/utils.py'),
                           'gnr.utils.kpss_test': ('utils.html#kpss_test', 'gnr/utils.py'),
                           'gnr.utils.make_stable': ('utils.html#make_stable', 'gnr/utils.py')}}}
